# coding=utf-8
import json
from django.db.models import Sum, F
from django.db import transaction
from django.db.models import Q
from rest_framework.views import APIView
from rest_framework.generics import ListAPIView
from utils.custom_modelviewset import CustomModelViewSet
from .serializers import *
from .filters import *
from apps.log.models import BizLog
from utils import response_ok, response_error
from utils.permission import isLogin
from apps.order.models import Order
from rest_framework.decorators import action
from django.contrib.auth import get_user_model

User = get_user_model()
from utils.format import strftime


class ProductView(APIView):

    def get(self, request):
        data = []
        queryset = Product.objects.filter(enable=True)
        for row in queryset:
            item = {
                'id': row.id,
                'name': row.name,
                'month': row.month,
            }
            data.append(item)
        return response_ok(data)

class DeleteProductImageView(APIView):
    permission_classes = [isLogin, ]

    def post(self, request, *args, **kwargs):
        # 商品图片 删除
        id = request.GET.get('id')
        try:
            instance = ProductImages.getById(id)
            image = Upload.objects.filter(id=instance.image_id).first()
            with transaction.atomic():
                instance.delete()
                image.del_images()
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))
        return response_ok()


class ProductViewSet(CustomModelViewSet):
    permission_classes = [isLogin, ]
    queryset = Product.objects.filter()
    serializer_class = ProductSerializer

    def filter_queryset(self, queryset):
        f = ProductFilter(self.request.GET, queryset=queryset)
        return f.qs

    @action(methods=['post'], detail=True)
    def upload_image(self, request, pk):
        banner_img = request.FILES.get('banner_img')
        try:
            with transaction.atomic():
                instance = self.get_object()
                upload = Upload.objects._addnew(banner_img, 'commodity_image')
                if upload:
                    if instance.main_image_id:
                        img = Upload.objects.filter(id=instance.main_image_id).first()
                        img.del_images()
                    instance.main_image_id = upload.id
                    instance.save()
                    return response_ok(upload.id)
                return response_error('上传失败，请重新上传！')
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))

    @action(methods=['post'], detail=True)
    def upload_detail_img(self, request, pk):
        # 上传详情、参数图片
        try:
            with transaction.atomic():
                instance = self.get_object()
                for i in range(10):
                    file = request.data.get('file{}'.format(i))
                    if file:
                        upload = Upload.objects._addnew(file, 'commodity_image')
                        ProductImages.objects.create(product_id=instance.id,
                                                     image_id=upload.id)

                        if not upload:
                            return response_error('上传失败，请重新上传！')
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))
        return response_ok()

    @action(methods=['post'], detail=True)
    def query_detail_img(self, request, pk):
        # 查看 详情图片
        instance = self.get_object()
        data = []

        commodity_images = ProductImages.objects.filter(product_id=instance.id)
        for commodity_image in commodity_images:
            image = Upload.objects.filter(id=commodity_image.image_id).first()
            image_path = image.get_path()
            commodity_image_item = {
                'id': commodity_image.id,
                'name': commodity_image.name,
                'src': image_path,
            }
            data.append(commodity_image_item)
        return response_ok(data)


class ProductListView(ListAPIView):
    queryset = Product.objects.filter()
    serializer_class = ProductListSerializer

    def filter_queryset(self, queryset, ):
        f = ProductFilter(self.request.GET, queryset=queryset)
        return f.qs


class OrderListView(ListAPIView):
    permission_classes = [isLogin, ]
    queryset = Order.objects.filter()
    serializer_class = OrderListSerializer

    def filter_queryset(self, queryset, ):
        queryset = queryset.filter(Q(create_user=self.request.user) | Q(seller=self.request.user))
        f = OrderFilter(self.request.GET, queryset=queryset)
        return f.qs


class OrderViewSet(CustomModelViewSet):
    permission_classes = [isLogin, ]
    queryset = Order.objects.filter()
    serializer_class = OrderSerializer

    def filter_queryset(self, queryset):
        queryset = queryset.filter(Q(create_user=self.request.user) | Q(seller=self.request.user))
        f = OrderFilter(self.request.GET, queryset=queryset)
        return f.qs

    def retrieve(self, request, *args, **kwargs):
        # 小程序草稿箱、已上报修改，调用明细
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        if instance.category == order.ZERO:
            return response_ok(serializer.data)
        else:
            # y已上报修改，返回需要修改的字段列表
            edit_data = []
            edit_values = OrderDetailEdit.objects.filter(order=instance, type=OrderDetailEdit.ZERO).values('value')
            for r in edit_values:
                item = {
                    'name': r['value'],
                    'reason': '填错了，请修改',
                }
                edit_data.append(item)
            result = {
                'data': serializer.data,
                'edit_values': edit_data,
            }
            return response_ok(result)

    def create(self, request, *args, **kwargs):
        try:
            with transaction.atomic():
                serializer = self.get_serializer(data=request.data)
                if serializer.is_valid(raise_exception=True):
                    instance = serializer.save()
                    BizLog.objects.addnew(self.request.user, BizLog.INSERT,
                                          u'添加订单[%s]，id=%d' % (instance.name, instance.id), request.data)
                    return response_ok()
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))

    def update(self, request, *args, **kwargs):
        data = request.POST
        try:
            instance = self.get_object()
            with transaction.atomic():
                serializer = self.get_serializer(instance, data=request.data)
                if serializer.is_valid(raise_exception=True):
                    instance = serializer.save()
                    # 已上报的，修改，记录修改时间
                    if instance.category == order.ONE:
                        for key in data:
                            edit = OrderDetailEdit.objects.filter(type=OrderDetailEdit.ZERO, order=instance,
                                                                  value=key).first()
                            if edit:
                                edit.edit_time = datetime.datetime.now()
                                edit.save()

                    BizLog.objects.addnew(self.request.user, BizLog.INSERT,
                                          u'修改订单[%s]，id=%d' % (instance.name, instance.id), request.data)
                    return response_ok()
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))

    @action(methods=['post'], detail=True)
    def checkEdit(self, request, pk):
        data = request.POST.get('items')
        status = request.GET.get('statu')
        try:
            instance = self.get_object()
            with transaction.atomic():
                if status == '1':
                    # 通过
                    instance.status = order.ONE
                elif status == '3':
                    # 拒绝审批
                    instance.status = order.THREE
                else:
                    OrderDetailEdit.objects.filter(order=instance).delete()
                    if not len(data):
                        raise CustomError('请填写返回修改原因')
                    for item in json.loads(data):
                        OrderDetailEdit.objects.create(order=instance, value=item['name'], reason=item['reason'],
                                                       type=item['type'])
                    instance.status = order.TOW
                instance.save()
                BizLog.objects.addnew(self.request.user, BizLog.INSERT,
                                      u'审批订单[%s]，id=%d' % (instance.name, instance.id), request.data)
                return response_ok()
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))

    @action(methods=['post'], detail=False)
    def upload_image(self, request):
        file = self.request.FILES.get('file')
        type = self.request.FILES.get('type')
        order_id = self.request.FILES.get('order_id')
        upload = Upload.objects._addnew(file)
        if upload:
            OrderImages.objects.create(order_id=order_id, type=type, image=upload)
            return response_ok(upload.id)
        return response_error('上传失败，请重新上传！')

    @action(methods=['get'], detail=True)
    def get_detail(self, request, pk):
        # 后端审核业务，获取客户信息和部分字段修改时间
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        # y已上报修改，返回需要修改的字段列表
        # 不显示字段
        show = [
            'vmodel', 'invioce', 'ptax', 'insurance', 'boutique', 'wextension', 'upkeep', 'dpratio', 'lamount',
            'laompany_text', 'name', 'sex_text', 'bdate', 'rpr_text', 'nationality_text', 'ntype_text', 'inumber',
            'dincome', 'mstatus_text',
            'elevel_text', 'naddress', 'hptype_text', 'maddress_text', 'email', 'wx', 'phone', 'tel', 'wname', 'wphone',
            'waddress', 'itype_text', 'otype_text', 'position', 'wbst_text', 'bstname', 'bstcode', 'bsttel',
            'bstaddress',
            'cname', 'ctel', 'relation_text', 'caddress', 'ctname', 'cttel', 'thesame_text', 'tsname', 'tsbdate',
            'tsitype_text', 'tsinumber', 'tstel', 'tsrelation_text', 'tsnaddress', 'tsmaddress', 'tsemail', 'tswx',
            'tswname', 'tswphone', 'tswaddress', 'tsmincome',
        ]
        result = []
        for s in show:
            key = serializer[s]
            edit_values = OrderDetailEdit.objects.filter(order=instance, type=OrderDetailEdit.ZERO,
                                                         value=key.name).first()
            edit_time = reason = ''
            if edit_values:
                edit_time = edit_values.edit_time and strftime(edit_values.edit_time) or ''
                reason = edit_values.reason
            item = {
                'name': key.name,  # tel
                'label': key.label,  # 电话
                'value': key.value,  # 13566
                'edit_time': edit_time,
                'reason': reason,
                'type': '0',
            }
            result.append(item)
        return response_ok(result)

    @action(methods=['post'], detail=True)
    def update_image(self, request):
        file = self.request.FILES.get('file')
        upload_id = self.request.FILES.get('upload_id')
        upload = Upload.objects.filter(id=upload_id).first()
        if upload:
            upload.del_images()
            Upload.objects._update(file, 'user_image', upload)
            return response_ok(upload.id)
        return response_error('更新失败，请重新上传！')

    def destroy(self, request, *args, **kwargs):
        try:
            instance = self.get_object()
            if instance.category == order.ONE:
                raise CustomError('已上报信息，禁止删除!')
            with transaction.atomic():
                OrderDetailEdit.objects.filter(order=instance).delete()
                images = OrderImages.objects.filter(order=instance)
                for image in images:
                    upload = Upload.objects.filter(id=image.image.id)
                    upload.del_images()
                images.delete()
                super(OrderViewSet, self).destroy(request, *args, **kwargs)
        except CustomError as e:
            return response_error(e.get_error_msg())
        except Exception as e:
            return response_error(str(e))
        return response_ok()
